# -*- coding: utf-8 -*-
"""
    Trade query
"""
import datetime as dt
from futu.common.utils import *
from futu.quote.quote_query import pack_pb_req

# 无数据时的值
NoneDataValue = 'N/A'

def is_HKTrade_order_status_finish(status):
    val = int(status)
    if val == 3 or val == 5 or val == 6 or val == 7:
        return True
    return False


def is_USTrade_order_status_finish(status):
    val = int(status)
    if val == 3 or val == 5 or val == 6 or val == 7:
        return True
    return False


class GetAccountList:
    """Get the trade account list"""
    def __init__(self):
        pass

    @classmethod
    def pack_req(cls, user_id, conn_id):
        from futu.common.pb.Trd_GetAccList_pb2 import Request

        req = Request()
        req.c2s.userID = user_id
        return pack_pb_req(req, ProtoId.Trd_GetAccList, conn_id)

    @classmethod
    def unpack_rsp(cls, rsp_pb):
        """Convert from PLS response to user response"""
        if rsp_pb.retType != RET_OK:
            return RET_ERROR, rsp_pb.retMsg, None

        raw_acc_list = rsp_pb.s2c.accList
        acc_list = [{
            'acc_id': record.accID,
            'trd_env': TrdEnv.to_string2(record.trdEnv),
            'trdMarket_list': [ TrdMarket.to_string2(trdMkt) for trdMkt in record.trdMarketAuthList],
            'acc_type': TrdAccType.to_string2(record.accType) if record.HasField("accType") else TrdAccType.NONE,
            'card_num': record.cardNum if record.HasField("cardNum") else "N/A",
            'security_firm': SecurityFirm.to_string2(record.securityFirm) if record.HasField('securityFirm') else SecurityFirm.NONE
        } for record in raw_acc_list]

        return RET_OK, "", acc_list


class UnlockTrade:
    """Unlock trade limitation lock"""
    def __init__(self):
        pass

    @classmethod
    def pack_req(cls, is_unlock, password_md5, conn_id):
        """Convert from user request for trading days to PLS request"""
        from futu.common.pb.Trd_UnlockTrade_pb2 import Request
        req = Request()
        req.c2s.unlock = is_unlock
        req.c2s.pwdMD5 = password_md5

        return pack_pb_req(req, ProtoId.Trd_UnlockTrade, conn_id)

    @classmethod
    def unpack_rsp(cls, rsp_pb):
        """Convert from PLS response to user response"""
        if rsp_pb.retType != RET_OK:
            return RET_ERROR, rsp_pb.retMsg, None

        if rsp_pb.HasField('retMsg'):
            return RET_OK, rsp_pb.retMsg, None
        return RET_OK, "", None


class SubAccPush:
    """sub acc push"""
    def __init__(self):
        pass

    @classmethod
    def pack_req(cls, acc_id_list, conn_id):
        from futu.common.pb.Trd_SubAccPush_pb2 import Request
        req = Request()
        for x in acc_id_list:
            req.c2s.accIDList.append(x)

        return pack_pb_req(req, ProtoId.Trd_SubAccPush, conn_id)

    @classmethod
    def unpack_rsp(cls, rsp_pb):
        """Convert from PLS response to user response"""
        if rsp_pb.retType != RET_OK:
            return RET_ERROR, rsp_pb.retMsg, None

        return RET_OK, "", None


class AccInfoQuery:
    """Class for querying information of account"""

    def __init__(self):
        pass

    @classmethod
    def pack_req(cls, acc_id, trd_market, trd_env, conn_id, refresh_cache, currency):
        from futu.common.pb.Trd_GetFunds_pb2 import Request
        req = Request()
        _, req.c2s.header.trdEnv = TrdEnv.to_number(trd_env)
        req.c2s.header.accID = acc_id
        _, req.c2s.header.trdMarket = TrdMarket.to_number(trd_market)
        if refresh_cache:
            req.c2s.refreshCache = refresh_cache
        req.c2s.currency = Currency.to_number(currency)[1]
        return pack_pb_req(req, ProtoId.Trd_GetFunds, conn_id)

    @classmethod
    def unpack_rsp(cls, rsp_pb):
        """Convert from PLS response to user response"""
        if rsp_pb.retType != RET_OK:
            return RET_ERROR, rsp_pb.retMsg, None

        raw_funds = rsp_pb.s2c.funds
        accinfo_list = [{
            'power': raw_funds.power,
            'total_assets': raw_funds.totalAssets,
            'cash': raw_funds.cash,
            'market_val': raw_funds.marketVal,
            'frozen_cash': raw_funds.frozenCash,
            'avl_withdrawal_cash': raw_funds.avlWithdrawalCash,
            'currency': Currency.to_string2(raw_funds.currency) if raw_funds.HasField('currency') else Currency.NONE,
            'available_funds': raw_funds.availableFunds if raw_funds.HasField('availableFunds') else NoneDataValue,
            'unrealized_pl': raw_funds.unrealizedPL if raw_funds.HasField('unrealizedPL') else NoneDataValue,
            'realized_pl': raw_funds.realizedPL if raw_funds.HasField('realizedPL') else NoneDataValue,
            'risk_level': CltRiskLevel.to_string2(raw_funds.riskLevel) if raw_funds.HasField('riskLevel') else CltRiskLevel.NONE,
            'initial_margin': raw_funds.initialMargin if raw_funds.HasField('initialMargin') else NoneDataValue,
            'maintenance_margin': raw_funds.maintenanceMargin if raw_funds.HasField('maintenanceMargin') else NoneDataValue,
            'hk_cash': NoneDataValue,
            'hk_avl_withdrawal_cash': NoneDataValue,
            'us_cash': NoneDataValue,
            'us_avl_withdrawal_cash': NoneDataValue
        }]
        for cashInfo in raw_funds.cashInfoList:
            if cashInfo.currency == Trd_Common_pb2.Currency_HKD:
                accinfo_list[0]['hk_cash'] = cashInfo.cash
                accinfo_list[0]['hk_avl_withdrawal_cash'] = cashInfo.availableBalance
            elif cashInfo.currency == Trd_Common_pb2.Currency_USD:
                accinfo_list[0]['us_cash'] = cashInfo.cash
                accinfo_list[0]['us_avl_withdrawal_cash'] = cashInfo.availableBalance
        return RET_OK, "", accinfo_list


class PositionListQuery:
    """Class for querying position list"""

    def __init__(self):
        pass

    @classmethod
    def pack_req(cls, code, pl_ratio_min,
                 pl_ratio_max, trd_env, acc_id, trd_mkt, conn_id, refresh_cache):
        """Convert from user request for trading days to PLS request"""
        from futu.common.pb.Trd_GetPositionList_pb2 import Request
        req = Request()
        _, req.c2s.header.trdEnv = TrdEnv.to_number(trd_env)
        req.c2s.header.accID = acc_id
        _, req.c2s.header.trdMarket = TrdMarket.to_number(trd_mkt)
        if code:
            req.c2s.filterConditions.codeList.append(code)
        if pl_ratio_min is not None:
            req.c2s.filterPLRatioMin = float(pl_ratio_min) / 100.0
        if pl_ratio_max is not None:
            req.c2s.filterPLRatioMax = float(pl_ratio_max) / 100.0
        if refresh_cache:
            req.c2s.refreshCache = refresh_cache

        return pack_pb_req(req, ProtoId.Trd_GetPositionList, conn_id)

    @classmethod
    def unpack_rsp(cls, rsp_pb):
        """Convert from PLS response to user response"""
        if rsp_pb.retType != RET_OK:
            return RET_ERROR, rsp_pb.retMsg, None

        raw_position_list = rsp_pb.s2c.positionList

        position_list = [{
                             "code": merge_trd_mkt_stock_str(position.secMarket, position.code),
                             "stock_name": position.name,
                             "qty": position.qty,
                             "can_sell_qty": position.canSellQty,
                             "cost_price": position.costPrice if position.HasField('costPrice') else NoneDataValue,
                             "cost_price_valid": position.HasField('costPrice'),
                             "market_val": position.val,
                             "nominal_price": position.price,
                             "pl_ratio": 100 * position.plRatio if position.HasField('plRatio') else NoneDataValue,
                             "pl_ratio_valid": position.HasField('plRatio'),
                             "pl_val": position.plVal,
                             "pl_val_valid": position.HasField('plVal'),
                             "today_buy_qty": position.td_buyQty if position.HasField('td_buyQty') else NoneDataValue,
                             "today_buy_val": position.td_buyVal if position.HasField('td_buyVal') else NoneDataValue,
                             "today_pl_val": position.td_plVal if position.HasField('td_plVal') else NoneDataValue,
                             "today_sell_qty": position.td_sellQty if position.HasField('td_sellQty') else NoneDataValue,
                             "today_sell_val": position.td_sellVal if position.HasField('td_sellVal') else NoneDataValue,
                             "position_side": PositionSide.to_string2(position.positionSide),
                             "unrealized_pl": position.unrealizedPL if position.HasField('unrealizedPL') else NoneDataValue,
                             "realized_pl": position.realizedPL if position.HasField('realizedPL') else NoneDataValue
                         } for position in raw_position_list]
        return RET_OK, "", position_list


class OrderListQuery:
    """Class for querying list queue"""
    def __init__(self):
        pass

    @classmethod
    def pack_req(cls, order_id, status_filter_list, code, start, end,
                 trd_env, acc_id, trd_mkt, conn_id, refresh_cache):
        """Convert from user request for trading days to PLS request"""
        from futu.common.pb.Trd_GetOrderList_pb2 import Request
        req = Request()
        _, req.c2s.header.trdEnv = TrdEnv.to_number(trd_env)
        req.c2s.header.accID = acc_id
        _, req.c2s.header.trdMarket = TrdMarket.to_number(trd_mkt)

        if code:
            req.c2s.filterConditions.codeList.append(code)
        if order_id:
            req.c2s.filterConditions.idList.append(int(order_id))

        if start:
            req.c2s.filterConditions.beginTime = start
        if end:
            req.c2s.filterConditions.endTime = end
        if refresh_cache:
            req.c2s.refreshCache = refresh_cache

        if len(status_filter_list):
            for order_status in status_filter_list:
                r, v = OrderStatus.to_number(order_status)
                if r:
                    req.c2s.filterStatusList.append(v)

        return pack_pb_req(req, ProtoId.Trd_GetOrderList, conn_id)

    @classmethod
    def parse_order(cls, rsp_pb, order):
        order_dict = {
            "code": merge_trd_mkt_stock_str(order.secMarket, order.code),
            "stock_name": order.name,
            "trd_side": TrdSide.to_string2(order.trdSide),
            "order_type": OrderType.to_string2(order.orderType),
            "order_status": OrderStatus.to_string2(order.orderStatus),
            "order_id": str(order.orderID),
            "qty": order.qty,
            "price": order.price,
            "create_time": order.createTime,
            "updated_time": order.updateTime,
            "dealt_qty": order.fillQty,
            "dealt_avg_price": order.fillAvgPrice,
            "last_err_msg": order.lastErrMsg,
            "remark": order.remark if order.HasField("remark") else "",
            "time_in_force": TimeInForce.to_string2(order.timeInForce),
            "fill_outside_rth": order.fillOutsideRTH if order.HasField("fillOutsideRTH") else 'N/A'
        }
        return order_dict

    @classmethod
    def unpack_rsp(cls, rsp_pb):
        """Convert from PLS response to user response"""
        if rsp_pb.retType != RET_OK:
            return RET_ERROR, rsp_pb.retMsg, None

        raw_order_list = rsp_pb.s2c.orderList
        order_list = [OrderListQuery.parse_order(rsp_pb, order) for order in raw_order_list]
        return RET_OK, "", order_list


class PlaceOrder:
    """Palce order class"""
    def __init__(self):
        pass

    @classmethod
    def pack_req(cls, trd_side, order_type, price, qty,
                 code, adjust_limit, trd_env, sec_mkt_str, acc_id, trd_mkt, conn_id, remark,
                 time_in_force, fill_outside_rth):
        """Convert from user request for place order to PLS request"""
        from futu.common.pb.Trd_PlaceOrder_pb2 import Request
        req = Request()
        serial_no = get_unique_id32()
        req.c2s.packetID.serialNo = serial_no
        req.c2s.packetID.connID = conn_id

        _, req.c2s.header.trdEnv = TrdEnv.to_number(trd_env)
        req.c2s.header.accID = acc_id
        _, req.c2s.header.trdMarket = TrdMarket.to_number(trd_mkt)

        _, req.c2s.trdSide = TrdSide.to_number(trd_side)
        _, req.c2s.orderType = OrderType.to_number(order_type)
        req.c2s.code = code
        req.c2s.qty = qty
        req.c2s.price = price
        req.c2s.adjustPrice = adjust_limit != 0
        req.c2s.adjustSideAndLimit = adjust_limit
        if remark is not None:
            req.c2s.remark = remark
        r, proto_qot_mkt = Market.to_number(sec_mkt_str)
        if not r:
            proto_qot_mkt = Qot_Common_pb2.QotMarket_Unknown
        proto_trd_sec_mkt = QOT_MARKET_TO_TRD_SEC_MARKET_MAP.get(proto_qot_mkt, Trd_Common_pb2.TrdSecMarket_Unknown)
        req.c2s.secMarket = proto_trd_sec_mkt
        ret, val = TimeInForce.to_number(time_in_force)
        if not ret:
            return RET_ERROR, val, None
        else:
            req.c2s.timeInForce = val

        req.c2s.fillOutsideRTH = fill_outside_rth

        return pack_pb_req(req, ProtoId.Trd_PlaceOrder, conn_id, serial_no)

    @classmethod
    def unpack_rsp(cls, rsp_pb):
        """Convert from PLS response to user response"""
        if rsp_pb.retType != RET_OK:
            return RET_ERROR, rsp_pb.retMsg, None

        order_id = str(rsp_pb.s2c.orderID)

        return RET_OK, "", order_id


class ModifyOrder:
    """modify order class"""
    def __init__(self):
        pass

    @classmethod
    def pack_req(cls, modify_order_op, order_id, price, qty,
                 adjust_limit, trd_env, acc_id, trd_mkt, conn_id):
        """Convert from user request for place order to PLS request"""
        from futu.common.pb.Trd_ModifyOrder_pb2 import Request
        req = Request()
        serial_no = get_unique_id32()
        req.c2s.packetID.serialNo = serial_no
        req.c2s.packetID.connID = conn_id

        _, req.c2s.header.trdEnv = TrdEnv.to_number(trd_env)
        req.c2s.header.accID = acc_id
        _, req.c2s.header.trdMarket = TrdMarket.to_number(trd_mkt)

        req.c2s.orderID = int(order_id)
        _, req.c2s.modifyOrderOp = ModifyOrderOp.to_number(modify_order_op)
        req.c2s.forAll = False

        if modify_order_op == ModifyOrderOp.NORMAL:
            req.c2s.qty = qty
            req.c2s.price = price
            req.c2s.adjustPrice = adjust_limit != 0
            req.c2s.adjustSideAndLimit = adjust_limit

        return pack_pb_req(req, ProtoId.Trd_ModifyOrder, conn_id, serial_no)

    @classmethod
    def unpack_rsp(cls, rsp_pb):
        """Convert from PLS response to user response"""
        if rsp_pb.retType != RET_OK:
            return RET_ERROR, rsp_pb.retMsg, None

        order_id = str(rsp_pb.s2c.orderID)
        modify_order_list = [{
            'trd_env': TrdEnv.to_string2(rsp_pb.s2c.header.trdEnv),
            'order_id': order_id
        }]

        return RET_OK, "", modify_order_list


class CancelOrder:
    """modify order class"""
    def __init__(self):
        pass

    @classmethod
    def pack_req(cls, trd_env, acc_id, trd_mkt, conn_id):
        """Convert from user request for place order to PLS request"""
        from futu.common.pb.Trd_ModifyOrder_pb2 import Request
        req = Request()
        serial_no = get_unique_id32()
        req.c2s.packetID.serialNo = serial_no
        req.c2s.packetID.connID = conn_id

        _, req.c2s.header.trdEnv = TrdEnv.to_number(trd_env)
        req.c2s.header.accID = acc_id
        _, req.c2s.header.trdMarket = TrdMarket.to_number(trd_mkt)

        req.c2s.orderID = 0
        req.c2s.modifyOrderOp = Trd_Common_pb2.ModifyOrderOp_Cancel
        req.c2s.forAll = True
        return pack_pb_req(req, ProtoId.Trd_ModifyOrder, conn_id, serial_no)

    @classmethod
    def unpack_rsp(cls, rsp_pb):
        """Convert from PLS response to user response"""
        if rsp_pb.retType != RET_OK:
            return RET_ERROR, rsp_pb.retMsg, None
        return RET_OK, "success", None


class DealListQuery:
    """Class for """
    def __init__(self):
        pass

    @classmethod
    def pack_req(cls, code, trd_env, acc_id, trd_mkt, conn_id, refresh_cache):
        """Convert from user request for place order to PLS request"""
        from futu.common.pb.Trd_GetOrderFillList_pb2 import Request
        req = Request()
        _, req.c2s.header.trdEnv = TrdEnv.to_number(trd_env)
        req.c2s.header.accID = acc_id
        _, req.c2s.header.trdMarket = TrdMarket.to_number(trd_mkt)

        if code:
            req.c2s.filterConditions.codeList.append(code)

        if refresh_cache:
            req.c2s.refreshCache = refresh_cache

        return pack_pb_req(req, ProtoId.Trd_GetOrderFillList, conn_id)

    @classmethod
    def parse_deal(cls, rsp_pb, deal):
        deal_dict = {
            "code": merge_trd_mkt_stock_str(deal.secMarket, deal.code),
            "stock_name": deal.name,
            "deal_id": deal.fillID,
            "order_id": str(deal.orderID) if deal.HasField('orderID') else NoneDataValue,
            "qty": deal.qty,
            "price": deal.price,
            "trd_side": TrdSide.to_string2(deal.trdSide),
            "create_time": deal.createTime,
            "counter_broker_id": deal.counterBrokerID if deal.HasField('counterBrokerID') else NoneDataValue,
            "counter_broker_name": deal.counterBrokerName if deal.HasField('counterBrokerName') else NoneDataValue,
            "status": DealStatus.to_string2(deal.status) if deal.HasField("status") else NoneDataValue
        }
        return deal_dict

    @classmethod
    def unpack_rsp(cls, rsp_pb):
        """Convert from PLS response to user response"""
        if rsp_pb.retType != RET_OK:
            return RET_ERROR, rsp_pb.retMsg, None

        raw_deal_list = rsp_pb.s2c.orderFillList
        deal_list = [DealListQuery.parse_deal(rsp_pb, deal) for deal in raw_deal_list]

        return RET_OK, "", deal_list


class HistoryOrderListQuery:
    """Class for querying Histroy Order"""

    def __init__(self):
        pass

    @classmethod
    def pack_req(cls, status_filter_list, code, start, end,
                 trd_env, acc_id, trd_mkt, conn_id):

        from futu.common.pb.Trd_GetHistoryOrderList_pb2 import Request
        req = Request()
        _, req.c2s.header.trdEnv = TrdEnv.to_number(trd_env)
        req.c2s.header.accID = acc_id
        _, req.c2s.header.trdMarket = TrdMarket.to_number(trd_mkt)

        if code:
            req.c2s.filterConditions.codeList.append(code)

        req.c2s.filterConditions.beginTime = start
        req.c2s.filterConditions.endTime = end

        if status_filter_list:
            for order_status in status_filter_list:
                r, v = OrderStatus.to_number(order_status)
                if r:
                    req.c2s.filterStatusList.append(v)

        return pack_pb_req(req, ProtoId.Trd_GetHistoryOrderList, conn_id)

    @classmethod
    def unpack_rsp(cls, rsp_pb):

        if rsp_pb.retType != RET_OK:
            return RET_ERROR, rsp_pb.retMsg, None

        raw_order_list = rsp_pb.s2c.orderList
        order_list = [{
                      "code": merge_trd_mkt_stock_str(order.secMarket, order.code),
                      "stock_name": order.name,
                      "trd_side": TrdSide.to_string2(order.trdSide),
                      "order_type": OrderType.to_string2(order.orderType),
                      "order_status": OrderStatus.to_string2(order.orderStatus),
                      "order_id": str(order.orderID),
                      "qty": order.qty,
                      "price": order.price,
                      "create_time": order.createTime,
                      "updated_time": order.updateTime,
                      "dealt_qty": order.fillQty,
                      "dealt_avg_price": order.fillAvgPrice,
                      "last_err_msg": order.lastErrMsg,
                      "remark": order.remark if order.HasField("remark") else "",
                      "time_in_force": TimeInForce.to_string2(order.timeInForce),
                      "fill_outside_rth": order.fillOutsideRTH if order.HasField("fillOutsideRTH") else 'N/A'
                      } for order in raw_order_list]
        return RET_OK, "", order_list


class HistoryDealListQuery:
    """Class for """

    def __init__(self):
        pass

    @classmethod
    def pack_req(cls, code, start, end, trd_env, acc_id, trd_mkt, conn_id):

        from futu.common.pb.Trd_GetHistoryOrderFillList_pb2 import Request
        req = Request()
        _, req.c2s.header.trdEnv = TrdEnv.to_number(trd_env)
        req.c2s.header.accID = acc_id
        _, req.c2s.header.trdMarket = TrdMarket.to_number(trd_mkt)

        if code:
            req.c2s.filterConditions.codeList.append(code)

        req.c2s.filterConditions.beginTime = start
        req.c2s.filterConditions.endTime = end

        return pack_pb_req(req, ProtoId.Trd_GetHistoryOrderFillList, conn_id)

    @classmethod
    def unpack_rsp(cls, rsp_pb):

        if rsp_pb.retType != RET_OK:
            return RET_ERROR, rsp_pb.retMsg, None

        raw_deal_list = rsp_pb.s2c.orderFillList
        deal_list = [{
                    "code": merge_trd_mkt_stock_str(deal.secMarket, deal.code),
                    "stock_name": deal.name,
                    "deal_id": deal.fillID,
                    "order_id": str(deal.orderID) if deal.HasField('orderID') else "",
                    "qty": deal.qty,
                    "price": deal.price,
                    "trd_side": TrdSide.to_string2(deal.trdSide),
                    "create_time": deal.createTime,
                    "counter_broker_id": deal.counterBrokerID if deal.HasField('counterBrokerID') else "",
                    "counter_broker_name": deal.counterBrokerName,
                    "status": DealStatus.to_string2(deal.status) if deal.HasField('status') else 'N/A'
                     } for deal in raw_deal_list]

        return RET_OK, "", deal_list


class UpdateOrderPush:
    """Class for order update push"""
    def __init__(self):
        pass

    @classmethod
    def unpack_rsp(cls, rsp_pb):
        if rsp_pb.retType != RET_OK:
            return RET_ERROR, rsp_pb.retMsg

        order_dict = OrderListQuery.parse_order(rsp_pb, rsp_pb.s2c.order)
        order_dict['trd_env'] = TrdEnv.to_string2(rsp_pb.s2c.header.trdEnv)
        order_dict['trd_market'] = TrdMarket.to_string2(rsp_pb.s2c.header.trdMarket)

        return RET_OK, order_dict


class UpdateDealPush:
    """Class for order update push"""
    def __init__(self):
        pass

    @classmethod
    def unpack_rsp(cls, rsp_pb):

        if rsp_pb.retType != RET_OK:
            return RET_ERROR, rsp_pb.retMsg

        deal_dict = DealListQuery.parse_deal(rsp_pb, rsp_pb.s2c.orderFill)
        deal_dict['trd_env'] = TrdEnv.to_string2(rsp_pb.s2c.header.trdEnv)
        deal_dict['trd_market'] = TrdMarket.to_string2(rsp_pb.s2c.header.trdMarket)

        return RET_OK, deal_dict


class AccTradingInfoQuery:
    def __init__(self):
        pass

    @classmethod
    def pack_req(cls, order_type, code, price, order_id, adjust_limit, sec_mkt_str, trd_env, acc_id, trd_mkt, conn_id):

        from futu.common.pb.Trd_GetMaxTrdQtys_pb2 import Request
        req = Request()
        _, req.c2s.header.trdEnv = TrdEnv.to_number(trd_env)
        req.c2s.header.accID = acc_id
        _, req.c2s.header.trdMarket = TrdMarket.to_number(trd_mkt)

        _, req.c2s.orderType = OrderType.to_number(order_type)
        req.c2s.code = code
        req.c2s.price = price
        if order_id is not None:
            req.c2s.orderID = int(order_id)
        if adjust_limit == 0:
            req.c2s.adjustPrice = False
        else:
            req.c2s.adjustPrice = True
            req.c2s.adjustSideAndLimit = adjust_limit

        r, proto_qot_mkt = Market.to_number(sec_mkt_str)
        if not r:
            proto_qot_mkt = Qot_Common_pb2.QotMarket_Unknown

        proto_trd_sec_mkt = QOT_MARKET_TO_TRD_SEC_MARKET_MAP.get(proto_qot_mkt, Trd_Common_pb2.TrdSecMarket_Unknown)
        req.c2s.secMarket = proto_trd_sec_mkt

        return pack_pb_req(req, ProtoId.Trd_GetMaxTrdQtys, conn_id)

    @classmethod
    def unpack_rsp(cls, rsp_pb):
        from futu.common.pb.Trd_Common_pb2 import MaxTrdQtys

        if rsp_pb.retType != RET_OK:
            return RET_ERROR, rsp_pb.retMsg, None

        info = rsp_pb.s2c.maxTrdQtys    # type: MaxTrdQtys
        data = [{
            'max_cash_buy': info.maxCashBuy,
            'max_cash_and_margin_buy': info.maxCashAndMarginBuy if info.HasField('maxCashAndMarginBuy') else NoneDataValue,
            'max_position_sell': info.maxPositionSell,
            'max_sell_short': info.maxSellShort if info.HasField('maxSellShort') else NoneDataValue,
            'max_buy_back': info.maxBuyBack if info.HasField('maxBuyBack') else NoneDataValue
        }]

        return RET_OK, "", data



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2017 Futu, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
from os.path import dirname, join
import sys

from setuptools import (
    find_packages,
    setup,
)

is_py2 = sys.version_info[0] == 2

with open(join(dirname(__file__), 'futu/VERSION.txt'), 'rb') as f:
    version = f.read().decode('ascii').strip()

install_requires = ["pandas",
                    "simplejson",
                    "protobuf==3.5.1",
                    "PyCryptodome",
                    ]

if is_py2:
    install_requires.append("selectors2")

setup(
    name='futu-api',
    version=version,
    description='Futu Quantitative Trading API',
    classifiers=[],
    keywords='Futu HK/US Stock Quant Trading API',
    author='Futu, Inc.',
    author_email='ftdev@futunn.com',
    url='https://github.com/FutunnOpen/py-futu-api',
    license='Apache License 2.0',
    packages=find_packages(exclude=[]),
    package_data={'': ['*.*']},
    include_package_data=True,
    install_requires=install_requires
)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: GetDelayStatistics.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import Common_pb2 as Common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='GetDelayStatistics.proto',
  package='GetDelayStatistics',
  syntax='proto2',
  serialized_pb=_b('\n\x18GetDelayStatistics.proto\x12\x12GetDelayStatistics\x1a\x0c\x43ommon.proto\"B\n\x03\x43\x32S\x12\x10\n\x08typeList\x18\x01 \x03(\x05\x12\x14\n\x0cqotPushStage\x18\x02 \x01(\x05\x12\x13\n\x0bsegmentList\x18\x03 \x03(\x05\"m\n\x13\x44\x65layStatisticsItem\x12\r\n\x05\x62\x65gin\x18\x01 \x02(\x05\x12\x0b\n\x03\x65nd\x18\x02 \x02(\x05\x12\r\n\x05\x63ount\x18\x03 \x02(\x05\x12\x12\n\nproportion\x18\x04 \x02(\x02\x12\x17\n\x0f\x63umulativeRatio\x18\x05 \x02(\x02\"\x82\x01\n\x0f\x44\x65layStatistics\x12\x13\n\x0bqotPushType\x18\x01 \x02(\x05\x12\x39\n\x08itemList\x18\x02 \x03(\x0b\x32\'.GetDelayStatistics.DelayStatisticsItem\x12\x10\n\x08\x64\x65layAvg\x18\x03 \x02(\x02\x12\r\n\x05\x63ount\x18\x04 \x02(\x05\"\x8f\x01\n\x16ReqReplyStatisticsItem\x12\x0f\n\x07protoID\x18\x01 \x02(\x05\x12\r\n\x05\x63ount\x18\x02 \x02(\x05\x12\x14\n\x0ctotalCostAvg\x18\x03 \x02(\x02\x12\x14\n\x0copenDCostAvg\x18\x04 \x02(\x02\x12\x13\n\x0bnetDelayAvg\x18\x05 \x02(\x02\x12\x14\n\x0cisLocalReply\x18\x06 \x02(\x08\"w\n\x18PlaceOrderStatisticsItem\x12\x0f\n\x07orderID\x18\x01 \x02(\t\x12\x11\n\ttotalCost\x18\x02 \x02(\x02\x12\x11\n\topenDCost\x18\x03 \x02(\x02\x12\x10\n\x08netDelay\x18\x04 \x02(\x02\x12\x12\n\nupdateCost\x18\x05 \x02(\x02\"\xe5\x01\n\x03S2C\x12\x42\n\x15qotPushStatisticsList\x18\x01 \x03(\x0b\x32#.GetDelayStatistics.DelayStatistics\x12J\n\x16reqReplyStatisticsList\x18\x02 \x03(\x0b\x32*.GetDelayStatistics.ReqReplyStatisticsItem\x12N\n\x18placeOrderStatisticsList\x18\x03 \x03(\x0b\x32,.GetDelayStatistics.PlaceOrderStatisticsItem\"/\n\x07Request\x12$\n\x03\x63\x32s\x18\x01 \x02(\x0b\x32\x17.GetDelayStatistics.C2S\"h\n\x08Response\x12\x15\n\x07retType\x18\x01 \x02(\x05:\x04-400\x12\x0e\n\x06retMsg\x18\x02 \x01(\t\x12\x0f\n\x07\x65rrCode\x18\x03 \x01(\x05\x12$\n\x03s2c\x18\x04 \x01(\x0b\x32\x17.GetDelayStatistics.S2C*\x9c\x01\n\x13\x44\x65layStatisticsType\x12\x1e\n\x1a\x44\x65layStatisticsType_Unkonw\x10\x00\x12\x1f\n\x1b\x44\x65layStatisticsType_QotPush\x10\x01\x12 \n\x1c\x44\x65layStatisticsType_ReqReply\x10\x02\x12\"\n\x1e\x44\x65layStatisticsType_PlaceOrder\x10\x03*\x9f\x01\n\x0cQotPushStage\x12\x17\n\x13QotPushStage_Unkonw\x10\x00\x12\x16\n\x12QotPushStage_SR2SS\x10\x01\x12\x16\n\x12QotPushStage_SS2CR\x10\x02\x12\x16\n\x12QotPushStage_CR2CS\x10\x03\x12\x16\n\x12QotPushStage_SS2CS\x10\x04\x12\x16\n\x12QotPushStage_SR2CS\x10\x05*\x87\x01\n\x0bQotPushType\x12\x16\n\x12QotPushType_Unkonw\x10\x00\x12\x15\n\x11QotPushType_Price\x10\x01\x12\x16\n\x12QotPushType_Ticker\x10\x02\x12\x19\n\x15QotPushType_OrderBook\x10\x03\x12\x16\n\x12QotPushType_Broker\x10\x04\x42I\n\x13\x63om.futu.openapi.pbZ2github.com/futuopen/ftapi4go/pb/getdelaystatistics')
  ,
  dependencies=[Common__pb2.DESCRIPTOR,])

_DELAYSTATISTICSTYPE = _descriptor.EnumDescriptor(
  name='DelayStatisticsType',
  full_name='GetDelayStatistics.DelayStatisticsType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='DelayStatisticsType_Unkonw', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DelayStatisticsType_QotPush', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DelayStatisticsType_ReqReply', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DelayStatisticsType_PlaceOrder', index=3, number=3,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1029,
  serialized_end=1185,
)
_sym_db.RegisterEnumDescriptor(_DELAYSTATISTICSTYPE)

DelayStatisticsType = enum_type_wrapper.EnumTypeWrapper(_DELAYSTATISTICSTYPE)
_QOTPUSHSTAGE = _descriptor.EnumDescriptor(
  name='QotPushStage',
  full_name='GetDelayStatistics.QotPushStage',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='QotPushStage_Unkonw', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='QotPushStage_SR2SS', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='QotPushStage_SS2CR', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='QotPushStage_CR2CS', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='QotPushStage_SS2CS', index=4, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='QotPushStage_SR2CS', index=5, number=5,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1188,
  serialized_end=1347,
)
_sym_db.RegisterEnumDescriptor(_QOTPUSHSTAGE)

QotPushStage = enum_type_wrapper.EnumTypeWrapper(_QOTPUSHSTAGE)
_QOTPUSHTYPE = _descriptor.EnumDescriptor(
  name='QotPushType',
  full_name='GetDelayStatistics.QotPushType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='QotPushType_Unkonw', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='QotPushType_Price', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='QotPushType_Ticker', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='QotPushType_OrderBook', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='QotPushType_Broker', index=4, number=4,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1350,
  serialized_end=1485,
)
_sym_db.RegisterEnumDescriptor(_QOTPUSHTYPE)

QotPushType = enum_type_wrapper.EnumTypeWrapper(_QOTPUSHTYPE)
DelayStatisticsType_Unkonw = 0
DelayStatisticsType_QotPush = 1
DelayStatisticsType_ReqReply = 2
DelayStatisticsType_PlaceOrder = 3
QotPushStage_Unkonw = 0
QotPushStage_SR2SS = 1
QotPushStage_SS2CR = 2
QotPushStage_CR2CS = 3
QotPushStage_SS2CS = 4
QotPushStage_SR2CS = 5
QotPushType_Unkonw = 0
QotPushType_Price = 1
QotPushType_Ticker = 2
QotPushType_OrderBook = 3
QotPushType_Broker = 4



_C2S = _descriptor.Descriptor(
  name='C2S',
  full_name='GetDelayStatistics.C2S',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='typeList', full_name='GetDelayStatistics.C2S.typeList', index=0,
      number=1, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='qotPushStage', full_name='GetDelayStatistics.C2S.qotPushStage', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='segmentList', full_name='GetDelayStatistics.C2S.segmentList', index=2,
      number=3, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=62,
  serialized_end=128,
)


_DELAYSTATISTICSITEM = _descriptor.Descriptor(
  name='DelayStatisticsItem',
  full_name='GetDelayStatistics.DelayStatisticsItem',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='begin', full_name='GetDelayStatistics.DelayStatisticsItem.begin', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='end', full_name='GetDelayStatistics.DelayStatisticsItem.end', index=1,
      number=2, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='count', full_name='GetDelayStatistics.DelayStatisticsItem.count', index=2,
      number=3, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='proportion', full_name='GetDelayStatistics.DelayStatisticsItem.proportion', index=3,
      number=4, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cumulativeRatio', full_name='GetDelayStatistics.DelayStatisticsItem.cumulativeRatio', index=4,
      number=5, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=130,
  serialized_end=239,
)


_DELAYSTATISTICS = _descriptor.Descriptor(
  name='DelayStatistics',
  full_name='GetDelayStatistics.DelayStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='qotPushType', full_name='GetDelayStatistics.DelayStatistics.qotPushType', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='itemList', full_name='GetDelayStatistics.DelayStatistics.itemList', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='delayAvg', full_name='GetDelayStatistics.DelayStatistics.delayAvg', index=2,
      number=3, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='count', full_name='GetDelayStatistics.DelayStatistics.count', index=3,
      number=4, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=242,
  serialized_end=372,
)


_REQREPLYSTATISTICSITEM = _descriptor.Descriptor(
  name='ReqReplyStatisticsItem',
  full_name='GetDelayStatistics.ReqReplyStatisticsItem',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='protoID', full_name='GetDelayStatistics.ReqReplyStatisticsItem.protoID', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='count', full_name='GetDelayStatistics.ReqReplyStatisticsItem.count', index=1,
      number=2, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='totalCostAvg', full_name='GetDelayStatistics.ReqReplyStatisticsItem.totalCostAvg', index=2,
      number=3, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='openDCostAvg', full_name='GetDelayStatistics.ReqReplyStatisticsItem.openDCostAvg', index=3,
      number=4, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='netDelayAvg', full_name='GetDelayStatistics.ReqReplyStatisticsItem.netDelayAvg', index=4,
      number=5, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='isLocalReply', full_name='GetDelayStatistics.ReqReplyStatisticsItem.isLocalReply', index=5,
      number=6, type=8, cpp_type=7, label=2,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=375,
  serialized_end=518,
)


_PLACEORDERSTATISTICSITEM = _descriptor.Descriptor(
  name='PlaceOrderStatisticsItem',
  full_name='GetDelayStatistics.PlaceOrderStatisticsItem',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='orderID', full_name='GetDelayStatistics.PlaceOrderStatisticsItem.orderID', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='totalCost', full_name='GetDelayStatistics.PlaceOrderStatisticsItem.totalCost', index=1,
      number=2, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='openDCost', full_name='GetDelayStatistics.PlaceOrderStatisticsItem.openDCost', index=2,
      number=3, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='netDelay', full_name='GetDelayStatistics.PlaceOrderStatisticsItem.netDelay', index=3,
      number=4, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='updateCost', full_name='GetDelayStatistics.PlaceOrderStatisticsItem.updateCost', index=4,
      number=5, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=520,
  serialized_end=639,
)


_S2C = _descriptor.Descriptor(
  name='S2C',
  full_name='GetDelayStatistics.S2C',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='qotPushStatisticsList', full_name='GetDelayStatistics.S2C.qotPushStatisticsList', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='reqReplyStatisticsList', full_name='GetDelayStatistics.S2C.reqReplyStatisticsList', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='placeOrderStatisticsList', full_name='GetDelayStatistics.S2C.placeOrderStatisticsList', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=642,
  serialized_end=871,
)


_REQUEST = _descriptor.Descriptor(
  name='Request',
  full_name='GetDelayStatistics.Request',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='c2s', full_name='GetDelayStatistics.Request.c2s', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=873,
  serialized_end=920,
)


_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='GetDelayStatistics.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='retType', full_name='GetDelayStatistics.Response.retType', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=True, default_value=-400,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='retMsg', full_name='GetDelayStatistics.Response.retMsg', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='errCode', full_name='GetDelayStatistics.Response.errCode', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='s2c', full_name='GetDelayStatistics.Response.s2c', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=922,
  serialized_end=1026,
)

_DELAYSTATISTICS.fields_by_name['itemList'].message_type = _DELAYSTATISTICSITEM
_S2C.fields_by_name['qotPushStatisticsList'].message_type = _DELAYSTATISTICS
_S2C.fields_by_name['reqReplyStatisticsList'].message_type = _REQREPLYSTATISTICSITEM
_S2C.fields_by_name['placeOrderStatisticsList'].message_type = _PLACEORDERSTATISTICSITEM
_REQUEST.fields_by_name['c2s'].message_type = _C2S
_RESPONSE.fields_by_name['s2c'].message_type = _S2C
DESCRIPTOR.message_types_by_name['C2S'] = _C2S
DESCRIPTOR.message_types_by_name['DelayStatisticsItem'] = _DELAYSTATISTICSITEM
DESCRIPTOR.message_types_by_name['DelayStatistics'] = _DELAYSTATISTICS
DESCRIPTOR.message_types_by_name['ReqReplyStatisticsItem'] = _REQREPLYSTATISTICSITEM
DESCRIPTOR.message_types_by_name['PlaceOrderStatisticsItem'] = _PLACEORDERSTATISTICSITEM
DESCRIPTOR.message_types_by_name['S2C'] = _S2C
DESCRIPTOR.message_types_by_name['Request'] = _REQUEST
DESCRIPTOR.message_types_by_name['Response'] = _RESPONSE
DESCRIPTOR.enum_types_by_name['DelayStatisticsType'] = _DELAYSTATISTICSTYPE
DESCRIPTOR.enum_types_by_name['QotPushStage'] = _QOTPUSHSTAGE
DESCRIPTOR.enum_types_by_name['QotPushType'] = _QOTPUSHTYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

C2S = _reflection.GeneratedProtocolMessageType('C2S', (_message.Message,), dict(
  DESCRIPTOR = _C2S,
  __module__ = 'GetDelayStatistics_pb2'
  # @@protoc_insertion_point(class_scope:GetDelayStatistics.C2S)
  ))
_sym_db.RegisterMessage(C2S)

DelayStatisticsItem = _reflection.GeneratedProtocolMessageType('DelayStatisticsItem', (_message.Message,), dict(
  DESCRIPTOR = _DELAYSTATISTICSITEM,
  __module__ = 'GetDelayStatistics_pb2'
  # @@protoc_insertion_point(class_scope:GetDelayStatistics.DelayStatisticsItem)
  ))
_sym_db.RegisterMessage(DelayStatisticsItem)

DelayStatistics = _reflection.GeneratedProtocolMessageType('DelayStatistics', (_message.Message,), dict(
  DESCRIPTOR = _DELAYSTATISTICS,
  __module__ = 'GetDelayStatistics_pb2'
  # @@protoc_insertion_point(class_scope:GetDelayStatistics.DelayStatistics)
  ))
_sym_db.RegisterMessage(DelayStatistics)

ReqReplyStatisticsItem = _reflection.GeneratedProtocolMessageType('ReqReplyStatisticsItem', (_message.Message,), dict(
  DESCRIPTOR = _REQREPLYSTATISTICSITEM,
  __module__ = 'GetDelayStatistics_pb2'
  # @@protoc_insertion_point(class_scope:GetDelayStatistics.ReqReplyStatisticsItem)
  ))
_sym_db.RegisterMessage(ReqReplyStatisticsItem)

PlaceOrderStatisticsItem = _reflection.GeneratedProtocolMessageType('PlaceOrderStatisticsItem', (_message.Message,), dict(
  DESCRIPTOR = _PLACEORDERSTATISTICSITEM,
  __module__ = 'GetDelayStatistics_pb2'
  # @@protoc_insertion_point(class_scope:GetDelayStatistics.PlaceOrderStatisticsItem)
  ))
_sym_db.RegisterMessage(PlaceOrderStatisticsItem)

S2C = _reflection.GeneratedProtocolMessageType('S2C', (_message.Message,), dict(
  DESCRIPTOR = _S2C,
  __module__ = 'GetDelayStatistics_pb2'
  # @@protoc_insertion_point(class_scope:GetDelayStatistics.S2C)
  ))
_sym_db.RegisterMessage(S2C)

Request = _reflection.GeneratedProtocolMessageType('Request', (_message.Message,), dict(
  DESCRIPTOR = _REQUEST,
  __module__ = 'GetDelayStatistics_pb2'
  # @@protoc_insertion_point(class_scope:GetDelayStatistics.Request)
  ))
_sym_db.RegisterMessage(Request)

Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSE,
  __module__ = 'GetDelayStatistics_pb2'
  # @@protoc_insertion_point(class_scope:GetDelayStatistics.Response)
  ))
_sym_db.RegisterMessage(Response)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\023com.futu.openapi.pbZ2github.com/futuopen/ftapi4go/pb/getdelaystatistics'))
# @@protoc_insertion_point(module_scope)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Qot_GetOptionChain.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import Common_pb2 as Common__pb2
import Qot_Common_pb2 as Qot__Common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='Qot_GetOptionChain.proto',
  package='Qot_GetOptionChain',
  syntax='proto2',
  serialized_pb=_b('\n\x18Qot_GetOptionChain.proto\x12\x12Qot_GetOptionChain\x1a\x0c\x43ommon.proto\x1a\x10Qot_Common.proto\"\x80\x03\n\nDataFilter\x12\x1c\n\x14impliedVolatilityMin\x18\x01 \x01(\x01\x12\x1c\n\x14impliedVolatilityMax\x18\x02 \x01(\x01\x12\x10\n\x08\x64\x65ltaMin\x18\x03 \x01(\x01\x12\x10\n\x08\x64\x65ltaMax\x18\x04 \x01(\x01\x12\x10\n\x08gammaMin\x18\x05 \x01(\x01\x12\x10\n\x08gammaMax\x18\x06 \x01(\x01\x12\x0f\n\x07vegaMin\x18\x07 \x01(\x01\x12\x0f\n\x07vegaMax\x18\x08 \x01(\x01\x12\x10\n\x08thetaMin\x18\t \x01(\x01\x12\x10\n\x08thetaMax\x18\n \x01(\x01\x12\x0e\n\x06rhoMin\x18\x0b \x01(\x01\x12\x0e\n\x06rhoMax\x18\x0c \x01(\x01\x12\x1a\n\x12netOpenInterestMin\x18\r \x01(\x01\x12\x1a\n\x12netOpenInterestMax\x18\x0e \x01(\x01\x12\x17\n\x0fopenInterestMin\x18\x0f \x01(\x01\x12\x17\n\x0fopenInterestMax\x18\x10 \x01(\x01\x12\x0e\n\x06volMin\x18\x11 \x01(\x01\x12\x0e\n\x06volMax\x18\x12 \x01(\x01\"\xbc\x01\n\x03\x43\x32S\x12#\n\x05owner\x18\x01 \x02(\x0b\x32\x14.Qot_Common.Security\x12\x17\n\x0findexOptionType\x18\x06 \x01(\x05\x12\x0c\n\x04type\x18\x02 \x01(\x05\x12\x11\n\tcondition\x18\x03 \x01(\x05\x12\x11\n\tbeginTime\x18\x04 \x02(\t\x12\x0f\n\x07\x65ndTime\x18\x05 \x02(\t\x12\x32\n\ndataFilter\x18\x07 \x01(\x0b\x32\x1e.Qot_GetOptionChain.DataFilter\"g\n\nOptionItem\x12,\n\x04\x63\x61ll\x18\x01 \x01(\x0b\x32\x1e.Qot_Common.SecurityStaticInfo\x12+\n\x03put\x18\x02 \x01(\x0b\x32\x1e.Qot_Common.SecurityStaticInfo\"j\n\x0bOptionChain\x12\x12\n\nstrikeTime\x18\x01 \x02(\t\x12.\n\x06option\x18\x02 \x03(\x0b\x32\x1e.Qot_GetOptionChain.OptionItem\x12\x17\n\x0fstrikeTimestamp\x18\x03 \x01(\x01\";\n\x03S2C\x12\x34\n\x0boptionChain\x18\x01 \x03(\x0b\x32\x1f.Qot_GetOptionChain.OptionChain\"/\n\x07Request\x12$\n\x03\x63\x32s\x18\x01 \x02(\x0b\x32\x17.Qot_GetOptionChain.C2S\"h\n\x08Response\x12\x15\n\x07retType\x18\x01 \x02(\x05:\x04-400\x12\x0e\n\x06retMsg\x18\x02 \x01(\t\x12\x0f\n\x07\x65rrCode\x18\x03 \x01(\x05\x12$\n\x03s2c\x18\x04 \x01(\x0b\x32\x17.Qot_GetOptionChain.S2C*b\n\x0eOptionCondType\x12\x19\n\x15OptionCondType_Unknow\x10\x00\x12\x19\n\x15OptionCondType_WithIn\x10\x01\x12\x1a\n\x16OptionCondType_Outside\x10\x02\x42H\n\x13\x63om.futu.openapi.pbZ1github.com/futuopen/ftapi4go/pb/qotgetoptionchain')
  ,
  dependencies=[Common__pb2.DESCRIPTOR,Qot__Common__pb2.DESCRIPTOR,])

_OPTIONCONDTYPE = _descriptor.EnumDescriptor(
  name='OptionCondType',
  full_name='Qot_GetOptionChain.OptionCondType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='OptionCondType_Unknow', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OptionCondType_WithIn', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OptionCondType_Outside', index=2, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1087,
  serialized_end=1185,
)
_sym_db.RegisterEnumDescriptor(_OPTIONCONDTYPE)

OptionCondType = enum_type_wrapper.EnumTypeWrapper(_OPTIONCONDTYPE)
OptionCondType_Unknow = 0
OptionCondType_WithIn = 1
OptionCondType_Outside = 2



_DATAFILTER = _descriptor.Descriptor(
  name='DataFilter',
  full_name='Qot_GetOptionChain.DataFilter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='impliedVolatilityMin', full_name='Qot_GetOptionChain.DataFilter.impliedVolatilityMin', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='impliedVolatilityMax', full_name='Qot_GetOptionChain.DataFilter.impliedVolatilityMax', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='deltaMin', full_name='Qot_GetOptionChain.DataFilter.deltaMin', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='deltaMax', full_name='Qot_GetOptionChain.DataFilter.deltaMax', index=3,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='gammaMin', full_name='Qot_GetOptionChain.DataFilter.gammaMin', index=4,
      number=5, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='gammaMax', full_name='Qot_GetOptionChain.DataFilter.gammaMax', index=5,
      number=6, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vegaMin', full_name='Qot_GetOptionChain.DataFilter.vegaMin', index=6,
      number=7, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vegaMax', full_name='Qot_GetOptionChain.DataFilter.vegaMax', index=7,
      number=8, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='thetaMin', full_name='Qot_GetOptionChain.DataFilter.thetaMin', index=8,
      number=9, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='thetaMax', full_name='Qot_GetOptionChain.DataFilter.thetaMax', index=9,
      number=10, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='rhoMin', full_name='Qot_GetOptionChain.DataFilter.rhoMin', index=10,
      number=11, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='rhoMax', full_name='Qot_GetOptionChain.DataFilter.rhoMax', index=11,
      number=12, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='netOpenInterestMin', full_name='Qot_GetOptionChain.DataFilter.netOpenInterestMin', index=12,
      number=13, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='netOpenInterestMax', full_name='Qot_GetOptionChain.DataFilter.netOpenInterestMax', index=13,
      number=14, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='openInterestMin', full_name='Qot_GetOptionChain.DataFilter.openInterestMin', index=14,
      number=15, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='openInterestMax', full_name='Qot_GetOptionChain.DataFilter.openInterestMax', index=15,
      number=16, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='volMin', full_name='Qot_GetOptionChain.DataFilter.volMin', index=16,
      number=17, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='volMax', full_name='Qot_GetOptionChain.DataFilter.volMax', index=17,
      number=18, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=81,
  serialized_end=465,
)


_C2S = _descriptor.Descriptor(
  name='C2S',
  full_name='Qot_GetOptionChain.C2S',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='owner', full_name='Qot_GetOptionChain.C2S.owner', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='indexOptionType', full_name='Qot_GetOptionChain.C2S.indexOptionType', index=1,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='Qot_GetOptionChain.C2S.type', index=2,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='condition', full_name='Qot_GetOptionChain.C2S.condition', index=3,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='beginTime', full_name='Qot_GetOptionChain.C2S.beginTime', index=4,
      number=4, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='endTime', full_name='Qot_GetOptionChain.C2S.endTime', index=5,
      number=5, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dataFilter', full_name='Qot_GetOptionChain.C2S.dataFilter', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=468,
  serialized_end=656,
)


_OPTIONITEM = _descriptor.Descriptor(
  name='OptionItem',
  full_name='Qot_GetOptionChain.OptionItem',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='call', full_name='Qot_GetOptionChain.OptionItem.call', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='put', full_name='Qot_GetOptionChain.OptionItem.put', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=658,
  serialized_end=761,
)


_OPTIONCHAIN = _descriptor.Descriptor(
  name='OptionChain',
  full_name='Qot_GetOptionChain.OptionChain',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='strikeTime', full_name='Qot_GetOptionChain.OptionChain.strikeTime', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='option', full_name='Qot_GetOptionChain.OptionChain.option', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='strikeTimestamp', full_name='Qot_GetOptionChain.OptionChain.strikeTimestamp', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=763,
  serialized_end=869,
)


_S2C = _descriptor.Descriptor(
  name='S2C',
  full_name='Qot_GetOptionChain.S2C',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='optionChain', full_name='Qot_GetOptionChain.S2C.optionChain', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=871,
  serialized_end=930,
)


_REQUEST = _descriptor.Descriptor(
  name='Request',
  full_name='Qot_GetOptionChain.Request',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='c2s', full_name='Qot_GetOptionChain.Request.c2s', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=932,
  serialized_end=979,
)


_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='Qot_GetOptionChain.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='retType', full_name='Qot_GetOptionChain.Response.retType', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=True, default_value=-400,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='retMsg', full_name='Qot_GetOptionChain.Response.retMsg', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='errCode', full_name='Qot_GetOptionChain.Response.errCode', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='s2c', full_name='Qot_GetOptionChain.Response.s2c', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=981,
  serialized_end=1085,
)

_C2S.fields_by_name['owner'].message_type = Qot__Common__pb2._SECURITY
_C2S.fields_by_name['dataFilter'].message_type = _DATAFILTER
_OPTIONITEM.fields_by_name['call'].message_type = Qot__Common__pb2._SECURITYSTATICINFO
_OPTIONITEM.fields_by_name['put'].message_type = Qot__Common__pb2._SECURITYSTATICINFO
_OPTIONCHAIN.fields_by_name['option'].message_type = _OPTIONITEM
_S2C.fields_by_name['optionChain'].message_type = _OPTIONCHAIN
_REQUEST.fields_by_name['c2s'].message_type = _C2S
_RESPONSE.fields_by_name['s2c'].message_type = _S2C
DESCRIPTOR.message_types_by_name['DataFilter'] = _DATAFILTER
DESCRIPTOR.message_types_by_name['C2S'] = _C2S
DESCRIPTOR.message_types_by_name['OptionItem'] = _OPTIONITEM
DESCRIPTOR.message_types_by_name['OptionChain'] = _OPTIONCHAIN
DESCRIPTOR.message_types_by_name['S2C'] = _S2C
DESCRIPTOR.message_types_by_name['Request'] = _REQUEST
DESCRIPTOR.message_types_by_name['Response'] = _RESPONSE
DESCRIPTOR.enum_types_by_name['OptionCondType'] = _OPTIONCONDTYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DataFilter = _reflection.GeneratedProtocolMessageType('DataFilter', (_message.Message,), dict(
  DESCRIPTOR = _DATAFILTER,
  __module__ = 'Qot_GetOptionChain_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetOptionChain.DataFilter)
  ))
_sym_db.RegisterMessage(DataFilter)

C2S = _reflection.GeneratedProtocolMessageType('C2S', (_message.Message,), dict(
  DESCRIPTOR = _C2S,
  __module__ = 'Qot_GetOptionChain_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetOptionChain.C2S)
  ))
_sym_db.RegisterMessage(C2S)

OptionItem = _reflection.GeneratedProtocolMessageType('OptionItem', (_message.Message,), dict(
  DESCRIPTOR = _OPTIONITEM,
  __module__ = 'Qot_GetOptionChain_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetOptionChain.OptionItem)
  ))
_sym_db.RegisterMessage(OptionItem)

OptionChain = _reflection.GeneratedProtocolMessageType('OptionChain', (_message.Message,), dict(
  DESCRIPTOR = _OPTIONCHAIN,
  __module__ = 'Qot_GetOptionChain_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetOptionChain.OptionChain)
  ))
_sym_db.RegisterMessage(OptionChain)

S2C = _reflection.GeneratedProtocolMessageType('S2C', (_message.Message,), dict(
  DESCRIPTOR = _S2C,
  __module__ = 'Qot_GetOptionChain_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetOptionChain.S2C)
  ))
_sym_db.RegisterMessage(S2C)

Request = _reflection.GeneratedProtocolMessageType('Request', (_message.Message,), dict(
  DESCRIPTOR = _REQUEST,
  __module__ = 'Qot_GetOptionChain_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetOptionChain.Request)
  ))
_sym_db.RegisterMessage(Request)

Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSE,
  __module__ = 'Qot_GetOptionChain_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetOptionChain.Response)
  ))
_sym_db.RegisterMessage(Response)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\023com.futu.openapi.pbZ1github.com/futuopen/ftapi4go/pb/qotgetoptionchain'))
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
import os
import sys
import traceback
from futu.common import bytes_utf8, IS_PY2, str_utf8
from futu.common.constant import *
from futu.common.ft_logger import logger
from Crypto.PublicKey import RSA
from Crypto.Cipher import PKCS1_v1_5 as Cipher_pkcs1
from Crypto import Random


class SysConfig(object):
    IS_PROTO_ENCRYPT = False                # api通讯协议是否加密
    INIT_RSA_FILE = ''                      # 初始连接协议用到的rsa private key file
    RSA_OBJ = None                          # ras加解密对象
    PROTO_FMT = None                        # 协议格式
    CLINET_ID = None                        # Client标识
    CLIENT_VER = None                       # Client ver
    ALL_THREAD_DAEMON = False               # 是否所有产生的线程都是daemon线程

    @classmethod
    def set_client_info(cls, client_id, client_ver):
        """
        ..  py:function:: set_client_info(cls, client_id, client_ver)

        设置调用api的客户端信息, 非必调接口

        :param client_id: str, 客户端标识
        :param client_ver: int, 客户端版本号
        :return: None

        :example:

        .. code:: python

         from futu import *
         SysConfig.set_client_info("MyFutuAPI", 0)
         quote_ctx = OpenQuoteContext(host='127.0.0.1', port=11111)
         quote_ctx.close()

        """

        SysConfig.CLINET_ID = client_id
        SysConfig.CLIENT_VER = client_ver

    @classmethod
    def get_client_id(cls):
        return SysConfig.CLINET_ID if SysConfig.CLINET_ID else DEFULAT_CLIENT_ID

    @classmethod
    def get_client_ver(cls):
        return SysConfig.CLIENT_VER if SysConfig.CLIENT_VER is not None else CLIENT_VERSION

    @classmethod
    def get_proto_fmt(cls):
        return SysConfig.PROTO_FMT if SysConfig.PROTO_FMT else DEFULAT_PROTO_FMT

    @classmethod
    def set_proto_fmt(cls, proto_fmt):
        """

        ..  py:function:: set_proto_fmt(cls, proto_fmt)

        设置通讯协议body格式, 目前支持Protobuf | Json两种格式, 非必调接口

        :param proto_fmt: ProtoFMT
        :return: None

        :example:

        .. code:: python

         from futu import *
         SysConfig.set_proto_fmt(ProtoFMT.Protobuf)
         quote_ctx = OpenQuoteContext(host='127.0.0.1', port=11111)
         quote_ctx.close()

        """
        fmt_list = [ProtoFMT.Protobuf, ProtoFMT.Json]

        if proto_fmt not in fmt_list:
            raise Exception("proto_fmt error")
        SysConfig.PROTO_FMT = proto_fmt

    @classmethod
    def enable_proto_encrypt(cls, is_encrypt):
        """
        ..  py:function:: enable_proto_encrypt(cls, is_encrypt)

        设置通讯协议是否加密, 网关客户端和api需配置相同的RSA私钥文件,在连接初始化成功后，网关会下发随机生成的AES 加密密钥

        :param is_encrypt: bool
        :return: None

        :example:

        .. code:: python

         from futu import *
         SysConfig.enable_proto_encrypt(True)
         SysConfig.set_init_rsa_file("conn_key.txt")   # rsa 私钥文件路径
         quote_ctx = OpenQuoteContext(host='127.0.0.1', port=11111)
         quote_ctx.close()

        """
        SysConfig.IS_PROTO_ENCRYPT = bool(is_encrypt)

    @classmethod
    def set_init_rsa_file(cls, file):
        """
        ..  py:function:: set_init_rsa_file(cls, file)

        设置RSA私钥文件, 要求1024位, 格式为PKCS#1

        :param file:  str, 文件路径
        :return: None

        :example:

        .. code:: python

         from futu import *
         SysConfig.enable_proto_encrypt(True)
         SysConfig.set_init_rsa_file("conn_key.txt")   # rsa 私钥文件路径
         quote_ctx = OpenQuoteContext(host='127.0.0.1', port=11111)
         quote_ctx.close()

        """
        SysConfig.INIT_RSA_FILE = str(file)
        SysConfig.RSA_OBJ = None
        RsaCrypt.CHIPPER = None

    @classmethod
    def get_init_rsa_obj(cls):
        """
        :return: str , private key for init connect protocol
        """

        if not SysConfig.RSA_OBJ:
            SysConfig._read_rsa_keys()

        return SysConfig.RSA_OBJ

    @classmethod
    def is_proto_encrypt(cls):
        """
        :return: bool
        """
        return SysConfig.IS_PROTO_ENCRYPT

    @classmethod
    def _read_rsa_keys(cls):
        file_path = SysConfig.INIT_RSA_FILE if SysConfig.INIT_RSA_FILE \
            else os.path.join(os.path.dirname(__file__), DEFAULT_INIT_PRI_KEY_FILE)

        try:
            f = open(file_path, 'rb')
            df = f.read()
            if type(df) is not str:
                df = str_utf8(df)

            rsa = RSA.importKey(df)
            pub_key = rsa.publickey().exportKey()
            if not pub_key:
                raise Exception("Illegal format of file content")

            SysConfig.RSA_OBJ = rsa

        except Exception as e:
            traceback.print_exc()
            err = sys.exc_info()[1]
            err_msg = "Fatal error occurred in getting proto key, detail:{}".format(err)
            logger.error(err_msg)
            raise Exception(err_msg)

    @classmethod
    def set_all_thread_daemon(cls, all_daemon):
        """
        设置是否所有内部创建的线程都是daemon线程
        :param all_daemon: bool
        :return:
        """
        SysConfig.ALL_THREAD_DAEMON = all_daemon

    @classmethod
    def get_all_thread_daemon(cls):
        return SysConfig.ALL_THREAD_DAEMON


class RsaCrypt(object):
    RANDOM_GENERATOR = Random.new().read
    CHIPPER = None
    @classmethod
    def encrypt(cls, data):
        if RsaCrypt.CHIPPER is None:
            rsa = SysConfig.get_init_rsa_obj()
            RsaCrypt.CHIPPER = Cipher_pkcs1.new(rsa)

        if type(data) is not bytes:
            data = bytes_utf8(str(data))

        # 单次加密串的长度最大为(key_size / 8) - 11
        # 1024 bit的证书用100， 2048 bit的证书用 200
        one_len = 100
        ret_data = b''
        for i in range(0, len(data), one_len):
            ret_data += RsaCrypt.CHIPPER.encrypt(data[i:i + one_len])
        return ret_data

    @classmethod
    def decrypt(cls, data):
        if RsaCrypt.CHIPPER is None:
            rsa = SysConfig.get_init_rsa_obj()
            RsaCrypt.CHIPPER = Cipher_pkcs1.new(rsa)

        # 1024 bit的证书用128，2048 bit证书用256位
        one_len = 128
        ret_data = b''

        # python2下需转成str类型,否则异常
        if IS_PY2:
            data = str(data)

        for i in range(0, len(data), one_len):
            ret_data += RsaCrypt.CHIPPER.decrypt(data[i:i + one_len], RsaCrypt.RANDOM_GENERATOR)

        return ret_data


"""
test_str = 'futu api' * 32
dt_encrypt = RsaCrypt.encrypt(test_str)
print(dt_encrypt)
dt_decrypt = RsaCrypt.decrypt(dt_encrypt)
print(dt_decrypt)
"""

"""
from Crypto.Cipher import AES
from binascii import b2a_hex, a2b_hex

key = b'0123456789abcdef'
cryptor = AES.new(key, AES.MODE_ECB, key)

src = b'123'
len_src = len(src)
add = 16 - (len_src % 16)
src = src
src2 = src + (b'\0' * add)

dst = cryptor.encrypt(src2)
hex_dst = b2a_hex(dst)
print(hex_dst)

src3 = cryptor.decrypt(dst)
print("len={} decrypt={}".format(len(src3), src3))
"""








# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Qot_GetIpoList.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import Common_pb2 as Common__pb2
import Qot_Common_pb2 as Qot__Common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='Qot_GetIpoList.proto',
  package='Qot_GetIpoList',
  syntax='proto2',
  serialized_pb=_b('\n\x14Qot_GetIpoList.proto\x12\x0eQot_GetIpoList\x1a\x0c\x43ommon.proto\x1a\x10Qot_Common.proto\"m\n\x0c\x42\x61sicIpoData\x12&\n\x08security\x18\x01 \x02(\x0b\x32\x14.Qot_Common.Security\x12\x0c\n\x04name\x18\x02 \x02(\t\x12\x10\n\x08listTime\x18\x03 \x01(\t\x12\x15\n\rlistTimestamp\x18\x04 \x01(\x01\"\xb9\x03\n\x0b\x43NIpoExData\x12\x11\n\tapplyCode\x18\x01 \x02(\t\x12\x11\n\tissueSize\x18\x02 \x02(\x03\x12\x17\n\x0fonlineIssueSize\x18\x03 \x02(\x03\x12\x17\n\x0f\x61pplyUpperLimit\x18\x04 \x02(\x03\x12\x1d\n\x15\x61pplyLimitMarketValue\x18\x05 \x02(\x03\x12\x1a\n\x12isEstimateIpoPrice\x18\x06 \x02(\x08\x12\x10\n\x08ipoPrice\x18\x07 \x02(\x01\x12\x16\n\x0eindustryPeRate\x18\x08 \x02(\x01\x12\x1e\n\x16isEstimateWinningRatio\x18\t \x02(\x08\x12\x14\n\x0cwinningRatio\x18\n \x02(\x01\x12\x13\n\x0bissuePeRate\x18\x0b \x02(\x01\x12\x11\n\tapplyTime\x18\x0c \x01(\t\x12\x16\n\x0e\x61pplyTimestamp\x18\r \x01(\x01\x12\x13\n\x0bwinningTime\x18\x0e \x01(\t\x12\x18\n\x10winningTimestamp\x18\x0f \x01(\x01\x12\x10\n\x08isHasWon\x18\x10 \x02(\x08\x12\x36\n\x0ewinningNumData\x18\x11 \x03(\x0b\x32\x1e.Qot_GetIpoList.WinningNumData\":\n\x0eWinningNumData\x12\x13\n\x0bwinningName\x18\x01 \x02(\t\x12\x13\n\x0bwinningInfo\x18\x02 \x02(\t\"\xbe\x01\n\x0bHKIpoExData\x12\x13\n\x0bipoPriceMin\x18\x01 \x02(\x01\x12\x13\n\x0bipoPriceMax\x18\x02 \x02(\x01\x12\x11\n\tlistPrice\x18\x03 \x02(\x01\x12\x0f\n\x07lotSize\x18\x04 \x02(\x05\x12\x15\n\rentrancePrice\x18\x05 \x02(\x01\x12\x19\n\x11isSubscribeStatus\x18\x06 \x02(\x08\x12\x14\n\x0c\x61pplyEndTime\x18\x07 \x01(\t\x12\x19\n\x11\x61pplyEndTimestamp\x18\x08 \x01(\x01\"J\n\x0bUSIpoExData\x12\x13\n\x0bipoPriceMin\x18\x01 \x02(\x01\x12\x13\n\x0bipoPriceMax\x18\x02 \x02(\x01\x12\x11\n\tissueSize\x18\x03 \x02(\x03\"\xc3\x01\n\x07IpoData\x12+\n\x05\x62\x61sic\x18\x01 \x02(\x0b\x32\x1c.Qot_GetIpoList.BasicIpoData\x12-\n\x08\x63nExData\x18\x02 \x01(\x0b\x32\x1b.Qot_GetIpoList.CNIpoExData\x12-\n\x08hkExData\x18\x03 \x01(\x0b\x32\x1b.Qot_GetIpoList.HKIpoExData\x12-\n\x08usExData\x18\x04 \x01(\x0b\x32\x1b.Qot_GetIpoList.USIpoExData\"\x15\n\x03\x43\x32S\x12\x0e\n\x06market\x18\x01 \x02(\x05\"/\n\x03S2C\x12(\n\x07ipoList\x18\x01 \x03(\x0b\x32\x17.Qot_GetIpoList.IpoData\"+\n\x07Request\x12 \n\x03\x63\x32s\x18\x01 \x02(\x0b\x32\x13.Qot_GetIpoList.C2S\"d\n\x08Response\x12\x15\n\x07retType\x18\x01 \x02(\x05:\x04-400\x12\x0e\n\x06retMsg\x18\x02 \x01(\t\x12\x0f\n\x07\x65rrCode\x18\x03 \x01(\x05\x12 \n\x03s2c\x18\x04 \x01(\x0b\x32\x13.Qot_GetIpoList.S2CBD\n\x13\x63om.futu.openapi.pbZ-github.com/futuopen/ftapi4go/pb/qotgetipolist')
  ,
  dependencies=[Common__pb2.DESCRIPTOR,Qot__Common__pb2.DESCRIPTOR,])




_BASICIPODATA = _descriptor.Descriptor(
  name='BasicIpoData',
  full_name='Qot_GetIpoList.BasicIpoData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='security', full_name='Qot_GetIpoList.BasicIpoData.security', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='Qot_GetIpoList.BasicIpoData.name', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='listTime', full_name='Qot_GetIpoList.BasicIpoData.listTime', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='listTimestamp', full_name='Qot_GetIpoList.BasicIpoData.listTimestamp', index=3,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=72,
  serialized_end=181,
)


_CNIPOEXDATA = _descriptor.Descriptor(
  name='CNIpoExData',
  full_name='Qot_GetIpoList.CNIpoExData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='applyCode', full_name='Qot_GetIpoList.CNIpoExData.applyCode', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='issueSize', full_name='Qot_GetIpoList.CNIpoExData.issueSize', index=1,
      number=2, type=3, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='onlineIssueSize', full_name='Qot_GetIpoList.CNIpoExData.onlineIssueSize', index=2,
      number=3, type=3, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='applyUpperLimit', full_name='Qot_GetIpoList.CNIpoExData.applyUpperLimit', index=3,
      number=4, type=3, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='applyLimitMarketValue', full_name='Qot_GetIpoList.CNIpoExData.applyLimitMarketValue', index=4,
      number=5, type=3, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='isEstimateIpoPrice', full_name='Qot_GetIpoList.CNIpoExData.isEstimateIpoPrice', index=5,
      number=6, type=8, cpp_type=7, label=2,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ipoPrice', full_name='Qot_GetIpoList.CNIpoExData.ipoPrice', index=6,
      number=7, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='industryPeRate', full_name='Qot_GetIpoList.CNIpoExData.industryPeRate', index=7,
      number=8, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='isEstimateWinningRatio', full_name='Qot_GetIpoList.CNIpoExData.isEstimateWinningRatio', index=8,
      number=9, type=8, cpp_type=7, label=2,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='winningRatio', full_name='Qot_GetIpoList.CNIpoExData.winningRatio', index=9,
      number=10, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='issuePeRate', full_name='Qot_GetIpoList.CNIpoExData.issuePeRate', index=10,
      number=11, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='applyTime', full_name='Qot_GetIpoList.CNIpoExData.applyTime', index=11,
      number=12, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='applyTimestamp', full_name='Qot_GetIpoList.CNIpoExData.applyTimestamp', index=12,
      number=13, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='winningTime', full_name='Qot_GetIpoList.CNIpoExData.winningTime', index=13,
      number=14, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='winningTimestamp', full_name='Qot_GetIpoList.CNIpoExData.winningTimestamp', index=14,
      number=15, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='isHasWon', full_name='Qot_GetIpoList.CNIpoExData.isHasWon', index=15,
      number=16, type=8, cpp_type=7, label=2,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='winningNumData', full_name='Qot_GetIpoList.CNIpoExData.winningNumData', index=16,
      number=17, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=184,
  serialized_end=625,
)


_WINNINGNUMDATA = _descriptor.Descriptor(
  name='WinningNumData',
  full_name='Qot_GetIpoList.WinningNumData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='winningName', full_name='Qot_GetIpoList.WinningNumData.winningName', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='winningInfo', full_name='Qot_GetIpoList.WinningNumData.winningInfo', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=627,
  serialized_end=685,
)


_HKIPOEXDATA = _descriptor.Descriptor(
  name='HKIpoExData',
  full_name='Qot_GetIpoList.HKIpoExData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ipoPriceMin', full_name='Qot_GetIpoList.HKIpoExData.ipoPriceMin', index=0,
      number=1, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ipoPriceMax', full_name='Qot_GetIpoList.HKIpoExData.ipoPriceMax', index=1,
      number=2, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='listPrice', full_name='Qot_GetIpoList.HKIpoExData.listPrice', index=2,
      number=3, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lotSize', full_name='Qot_GetIpoList.HKIpoExData.lotSize', index=3,
      number=4, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='entrancePrice', full_name='Qot_GetIpoList.HKIpoExData.entrancePrice', index=4,
      number=5, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='isSubscribeStatus', full_name='Qot_GetIpoList.HKIpoExData.isSubscribeStatus', index=5,
      number=6, type=8, cpp_type=7, label=2,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='applyEndTime', full_name='Qot_GetIpoList.HKIpoExData.applyEndTime', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='applyEndTimestamp', full_name='Qot_GetIpoList.HKIpoExData.applyEndTimestamp', index=7,
      number=8, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=688,
  serialized_end=878,
)


_USIPOEXDATA = _descriptor.Descriptor(
  name='USIpoExData',
  full_name='Qot_GetIpoList.USIpoExData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ipoPriceMin', full_name='Qot_GetIpoList.USIpoExData.ipoPriceMin', index=0,
      number=1, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ipoPriceMax', full_name='Qot_GetIpoList.USIpoExData.ipoPriceMax', index=1,
      number=2, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='issueSize', full_name='Qot_GetIpoList.USIpoExData.issueSize', index=2,
      number=3, type=3, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=880,
  serialized_end=954,
)


_IPODATA = _descriptor.Descriptor(
  name='IpoData',
  full_name='Qot_GetIpoList.IpoData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='basic', full_name='Qot_GetIpoList.IpoData.basic', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cnExData', full_name='Qot_GetIpoList.IpoData.cnExData', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hkExData', full_name='Qot_GetIpoList.IpoData.hkExData', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='usExData', full_name='Qot_GetIpoList.IpoData.usExData', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=957,
  serialized_end=1152,
)


_C2S = _descriptor.Descriptor(
  name='C2S',
  full_name='Qot_GetIpoList.C2S',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='market', full_name='Qot_GetIpoList.C2S.market', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1154,
  serialized_end=1175,
)


_S2C = _descriptor.Descriptor(
  name='S2C',
  full_name='Qot_GetIpoList.S2C',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ipoList', full_name='Qot_GetIpoList.S2C.ipoList', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1177,
  serialized_end=1224,
)


_REQUEST = _descriptor.Descriptor(
  name='Request',
  full_name='Qot_GetIpoList.Request',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='c2s', full_name='Qot_GetIpoList.Request.c2s', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1226,
  serialized_end=1269,
)


_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='Qot_GetIpoList.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='retType', full_name='Qot_GetIpoList.Response.retType', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=True, default_value=-400,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='retMsg', full_name='Qot_GetIpoList.Response.retMsg', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='errCode', full_name='Qot_GetIpoList.Response.errCode', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='s2c', full_name='Qot_GetIpoList.Response.s2c', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1271,
  serialized_end=1371,
)

_BASICIPODATA.fields_by_name['security'].message_type = Qot__Common__pb2._SECURITY
_CNIPOEXDATA.fields_by_name['winningNumData'].message_type = _WINNINGNUMDATA
_IPODATA.fields_by_name['basic'].message_type = _BASICIPODATA
_IPODATA.fields_by_name['cnExData'].message_type = _CNIPOEXDATA
_IPODATA.fields_by_name['hkExData'].message_type = _HKIPOEXDATA
_IPODATA.fields_by_name['usExData'].message_type = _USIPOEXDATA
_S2C.fields_by_name['ipoList'].message_type = _IPODATA
_REQUEST.fields_by_name['c2s'].message_type = _C2S
_RESPONSE.fields_by_name['s2c'].message_type = _S2C
DESCRIPTOR.message_types_by_name['BasicIpoData'] = _BASICIPODATA
DESCRIPTOR.message_types_by_name['CNIpoExData'] = _CNIPOEXDATA
DESCRIPTOR.message_types_by_name['WinningNumData'] = _WINNINGNUMDATA
DESCRIPTOR.message_types_by_name['HKIpoExData'] = _HKIPOEXDATA
DESCRIPTOR.message_types_by_name['USIpoExData'] = _USIPOEXDATA
DESCRIPTOR.message_types_by_name['IpoData'] = _IPODATA
DESCRIPTOR.message_types_by_name['C2S'] = _C2S
DESCRIPTOR.message_types_by_name['S2C'] = _S2C
DESCRIPTOR.message_types_by_name['Request'] = _REQUEST
DESCRIPTOR.message_types_by_name['Response'] = _RESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

BasicIpoData = _reflection.GeneratedProtocolMessageType('BasicIpoData', (_message.Message,), dict(
  DESCRIPTOR = _BASICIPODATA,
  __module__ = 'Qot_GetIpoList_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetIpoList.BasicIpoData)
  ))
_sym_db.RegisterMessage(BasicIpoData)

CNIpoExData = _reflection.GeneratedProtocolMessageType('CNIpoExData', (_message.Message,), dict(
  DESCRIPTOR = _CNIPOEXDATA,
  __module__ = 'Qot_GetIpoList_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetIpoList.CNIpoExData)
  ))
_sym_db.RegisterMessage(CNIpoExData)

WinningNumData = _reflection.GeneratedProtocolMessageType('WinningNumData', (_message.Message,), dict(
  DESCRIPTOR = _WINNINGNUMDATA,
  __module__ = 'Qot_GetIpoList_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetIpoList.WinningNumData)
  ))
_sym_db.RegisterMessage(WinningNumData)

HKIpoExData = _reflection.GeneratedProtocolMessageType('HKIpoExData', (_message.Message,), dict(
  DESCRIPTOR = _HKIPOEXDATA,
  __module__ = 'Qot_GetIpoList_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetIpoList.HKIpoExData)
  ))
_sym_db.RegisterMessage(HKIpoExData)

USIpoExData = _reflection.GeneratedProtocolMessageType('USIpoExData', (_message.Message,), dict(
  DESCRIPTOR = _USIPOEXDATA,
  __module__ = 'Qot_GetIpoList_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetIpoList.USIpoExData)
  ))
_sym_db.RegisterMessage(USIpoExData)

IpoData = _reflection.GeneratedProtocolMessageType('IpoData', (_message.Message,), dict(
  DESCRIPTOR = _IPODATA,
  __module__ = 'Qot_GetIpoList_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetIpoList.IpoData)
  ))
_sym_db.RegisterMessage(IpoData)

C2S = _reflection.GeneratedProtocolMessageType('C2S', (_message.Message,), dict(
  DESCRIPTOR = _C2S,
  __module__ = 'Qot_GetIpoList_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetIpoList.C2S)
  ))
_sym_db.RegisterMessage(C2S)

S2C = _reflection.GeneratedProtocolMessageType('S2C', (_message.Message,), dict(
  DESCRIPTOR = _S2C,
  __module__ = 'Qot_GetIpoList_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetIpoList.S2C)
  ))
_sym_db.RegisterMessage(S2C)

Request = _reflection.GeneratedProtocolMessageType('Request', (_message.Message,), dict(
  DESCRIPTOR = _REQUEST,
  __module__ = 'Qot_GetIpoList_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetIpoList.Request)
  ))
_sym_db.RegisterMessage(Request)

Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSE,
  __module__ = 'Qot_GetIpoList_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetIpoList.Response)
  ))
_sym_db.RegisterMessage(Response)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\023com.futu.openapi.pbZ-github.com/futuopen/ftapi4go/pb/qotgetipolist'))
# @@protoc_insertion_point(module_scope)
